@@ -183,6 +183,9 @@ class Job:
                 build_script_pre=build_script_pre,
                 build_script_post=build_script_post,
             )
+            if entrypoint is not None:
+                image.path_to_repo = entrypoint.path_to_repo
+                image.path_to_module = entrypoint.path_to_module
         self.image = image
         self.type = type if entrypoint is None else entrypoint.type
         self.entrypoint = entrypoint
@@ -290,14 +293,30 @@ class Job:
         Returns a tuple of (branch: str, commit_hash: str).
         """
         path_to_repo = self.image.path_to_repo or "."
-        local_commit = run_shell_command(f"cd {path_to_repo} && git rev-parse HEAD")[0]
-        branch_name = run_shell_command(
-            f"cd {path_to_repo} && git rev-parse --abbrev-ref HEAD"
-        )[0]
+        try:
+            local_commit = run_shell_command(
+                f"cd {path_to_repo} && git rev-parse HEAD"
+            )[0]
+            branch_name = run_shell_command(
+                f"cd {path_to_repo} && git rev-parse --abbrev-ref HEAD"
+            )[0]
+            return_code = run_shell_command(
+                f"cd {path_to_repo} && git diff --exit-code"
+            )
+        except RuntimeError as e:
+            if "git: not found" in str(e):
+                logger.warning(
+                    "Git is not installed or not found in PATH. Repository validation is not possible."
+                )
+                branch_name, local_commit, return_code = "unknown", "unknown", None
+            else:
+                raise RepoValidationError(
+                    f"Error accessing git repository at {path_to_repo}: {e}"
+                ) from e
 
         if os.getenv("DATATAILR_ALLOW_UNSAFE_SCHEDULING", "false").lower() == "true":
             return branch_name, local_commit
-        return_code = run_shell_command(f"cd {path_to_repo} && git diff --exit-code")
+
         is_committed = return_code is not None and return_code[1] == 0
 
         if not is_committed:
@@ -337,6 +356,13 @@ class Job:
         """
         return {}
 
+    def __set_existing_id__(self, job_id: dict):
+        if isinstance(job_id, dict) and "Ids" in job_id and len(job_id["Ids"]) > 0:
+            job_id = job_id["Ids"][0]
+            if isinstance(job_id, str):
+                logger.debug(f"Setting existing job ID: {job_id}")
+                self.__id = job_id
+
     def __run_command__(self, command: str) -> Tuple[bool, str]:
         """
         Run a command in the context of the job.
@@ -350,18 +376,23 @@ class Job:
             temp_file_name = self.__prepare__()
 
             if command == "run":
-                __client__.run(f"file://{temp_file_name}", **self.get_schedule_args())
+                result = __client__.run(
+                    f"file://{temp_file_name}", **self.get_schedule_args()
+                )
             elif command == "save":
-                __client__.save(f"file://{temp_file_name}", **self.get_schedule_args())
+                result = __client__.save(
+                    f"file://{temp_file_name}", **self.get_schedule_args()
+                )
             elif command == "start":
-                __client__.start(self.name, environment=self.environment)
+                result = __client__.start(self.name, environment=self.environment)
             else:
                 raise ValueError(f"Unknown command: {command}")
             os.remove(temp_file_name)
         except Exception as e:
             logger.error(f"Error running command '{command}': {e}")
             return False, str(e)
-        return True, f"Job '{self.name}' {command}d successfully."
+        self.__set_existing_id__(result)
+        return True, result
 
     def save(self) -> Tuple[bool, str]:
         """
@@ -387,18 +418,31 @@ class Job:
 
     def promote(
         self,
-        from_environment: Optional[Environment] = None,
+        from_environment: Optional[Environment | str] = None,
         version: Optional[str | int] = None,
     ) -> Tuple[bool, str]:
         """
         Promote the job to the next environment.
         This method is used to promote a version of the job from one environment to the next one.
         If none of the environments to promote from are specified, it defaults to promote from all environments.
+
+        :param from_environment: The environment to promote from. If None, it will promote from all environments.
+        :param version: The version to promote. If None, it will promote the latest version.
+        :return: A tuple of (success: bool, message: str).
+
+        >>> from datatailr import Environment
+        >>> from datatailr.scheduler import Job
+        >>> job = Job(name="my_job", environment=Environment.DEV)
+        >>> job.promote(from_environment=Environment.DEV, version=3)
+
+        This will promote version 3 of the job from the DEV environment to the next environment (PRE).
         """
         promote_kwargs = {}
         if version is not None:
             promote_kwargs["version"] = str(version)
         if from_environment is not None:
+            if isinstance(from_environment, str):
+                from_environment = Environment(from_environment.lower())
             promote_kwargs["environment"] = str(from_environment)
         try:
             __client__.promote(self.name, **promote_kwargs)
