@@ -294,7 +294,7 @@ class BatchJob:
             env = {
                 "DATATAILR_BATCH_ID": str(self.dag.id),
                 "DATATAILR_JOB_ID": str(self.__id),
-                "DATATAILR_JOB_NAME": self.name,
+                "DATATAILR_JOB_NAME": f"{self.dag.name}[{self.__id}]",
             }
             self.entrypoint(env=env)
         else:
@@ -317,6 +317,7 @@ class Batch(Job):
         run_as: Optional[Union[str, User]] = None,
         resources: Resources = Resources(memory="100m", cpu=1),
         acl: Optional[ACL] = None,
+        python_version: str = "3.12",
         local_run: bool = False,
         python_requirements: str = "",
         build_script_pre: str = "",
@@ -330,6 +331,7 @@ class Batch(Job):
             run_as=run_as,
             resources=resources,
             acl=acl,
+            python_version=python_version,
             python_requirements=python_requirements,
             build_script_pre=build_script_pre,
             build_script_post=build_script_post,
@@ -477,10 +479,26 @@ class Batch(Job):
 
     def prepare_args(self) -> None:
         def arg_name(arg: Union[BatchJob, str]) -> str:
-            return arg.name if isinstance(arg, BatchJob) else arg
+            return f"{self.name}[{arg.id}]" if isinstance(arg, BatchJob) else arg
+
+        def adjust_mapping(mapping: Dict[str, str]) -> Dict[str, str]:
+            result = {}
+            for k, v in mapping.items():
+                if isinstance(v, BatchJob):
+                    result[k] = f"{self.name}[{v.id}]"
+                elif isinstance(v, str):
+                    job = self.get_job_by_name(v)
+                    if job is not None:
+                        result[k] = f"{self.name}[{job.id}]"
+                    else:
+                        result[k] = v
+                else:
+                    raise TypeError(
+                        f"Unsupported type in argument mapping: {type(v)} for key {k}"
+                    )
+            return result
 
         def merged(dst: dict[str, str], src: dict[str, str]) -> dict[str, str]:
-            # copy so we don't mutate the original mapping
             out = dict(dst)
             seen_vals = set(out.values())
             for k, v in src.items():
@@ -490,17 +508,19 @@ class Batch(Job):
             return out
 
         args = {
-            j.name: merged(
-                j.argument_mapping, {k: arg_name(v) for k, v in j.args.items()}
+            f"{self.name}[{j.id}]": merged(
+                adjust_mapping(j.argument_mapping),
+                {j.argument_mapping.get(k, k): arg_name(v) for k, v in j.args.items()},
             )
             for j in self.__jobs
         }
-
         __ARGUMENTS_CACHE__.add_arguments(self.id, args)
 
     def save(self) -> Tuple[bool, str]:
-        self.prepare_args()
-        return super().save()
+        status = super().save()
+        if status[0]:
+            self.prepare_args()
+        return status
 
     def run(self) -> Tuple[bool, str]:
         self.prepare_args()
