@@ -32,10 +32,13 @@
 #     DATATAILR_JOB_ID - the unique identifier for the job.
 
 
+import concurrent.futures
 import subprocess
 import os
-import sys
-from typing import Tuple
+import stat
+import shlex
+import sysconfig
+from typing import Optional, Tuple
 from datatailr.logging import DatatailrLogger
 from datatailr.utils import is_dt_installed
 
@@ -78,39 +81,105 @@ def create_user_and_group() -> Tuple[str, str]:
     os.system(
         f"getent passwd {user} || useradd -g {group} -s /bin/bash -m {user} -u {uid} -o"
     )
+
+    permissions = (
+        stat.S_IWOTH
+        | stat.S_IXOTH
+        | stat.S_IWUSR
+        | stat.S_IRUSR
+        | stat.S_IRGRP
+        | stat.S_IWGRP
+        | stat.S_IXUSR
+        | stat.S_IXGRP
+    )
+
+    os.makedirs(f"/home/{user}/tmp/.dt", exist_ok=True)
+    os.chmod(f"/home/{user}/tmp/.dt", permissions)
+
     return user, group
 
 
-def run_command_as_user(command: str | list, user: str, env_vars: dict):
-    """
-    Run a command as a specific user with the given environment variables.
-    """
-    if os.path.isfile("/etc/python-site-location"):
-        with open("/etc/python-site-location") as f:
-            python_site = f.read().rstrip()
-    else:
-        python_site = ""
+def prepare_command_argv(command: str | list, user: str, env_vars: dict) -> list[str]:
+    if isinstance(command, str):
+        command = shlex.split(command)
 
-    if isinstance(command, list):
-        command = " ".join(command)
-    env_vars = {
+    python_libdir = sysconfig.get_config_var("LIBDIR")
+    ld_library_path = get_env_var("LD_LIBRARY_PATH", None)
+
+    # Base environment variables setup
+    base_env = {
         "PATH": get_env_var("PATH", ""),
-        "PYTHONPATH": get_env_var("PYTHONPATH", "") + ":" + python_site,
-    } | env_vars
+        "PYTHONPATH": get_env_var("PYTHONPATH", ""),
+        "LD_LIBRARY_PATH": ":".join(filter(None, [python_libdir, ld_library_path])),
+    }
 
-    env_kv = [f"{k}={v}" for k, v in env_vars.items()]
-    argv = ["sudo", "-u", user, "env", *env_kv, "bash", "-lc", command]
+    merged_env = base_env | env_vars
+    env_kv = [f"{k}={v}" for k, v in merged_env.items()]
+    return ["sudo", "-u", user, "env", *env_kv, *command]
+
+
+def run_single_command_non_blocking(
+    command: str | list,
+    user: str,
+    env_vars: dict,
+    log_stream_name: Optional[str | None] = None,
+) -> int:
+    """
+    Runs a single command non-blocking and returns the exit code after it finishes.
+    This is designed to be run within an Executor.
+    """
+    argv = prepare_command_argv(command, user, env_vars)
+    cmd_label = " ".join(argv[4:])  # For logging purposes
 
     try:
-        proc = subprocess.Popen(argv)
+        if log_stream_name:
+            stdout_file_path = f"/opt/datatailr/var/log/{log_stream_name}.log"
+            stderr_file_path = f"/opt/datatailr/var/log/{log_stream_name}_error.log"
+            with (
+                open(stdout_file_path, "ab", buffering=0) as stdout_file,
+                open(stderr_file_path, "ab", buffering=0) as stderr_file,
+            ):
+                proc = subprocess.Popen(argv, stdout=stdout_file, stderr=stderr_file)
+        else:
+            proc = subprocess.Popen(argv)
         returncode = proc.wait()
+
         if returncode != 0:
-            logger.error(f"Command failed with exit code {returncode}")
-            sys.exit(returncode)
-    except subprocess.CalledProcessError as e:
-        logger.error(f"Command failed with exit code {e.returncode}")
-        logger.error(f"stderr: {e.stderr}")
-        sys.exit(1)
+            logger.error(f"Command '{cmd_label}' failed with exit code {returncode}")
+        return returncode
+    except Exception as e:
+        logger.error(f"Execution error for '{cmd_label}': {e}")
+        return 1
+
+
+def run_commands_in_parallel(
+    commands: list[str | list],
+    user: str,
+    env_vars: dict,
+    log_stream_names: Optional[list[str | None]] = None,
+) -> int:
+    """
+    Executes two commands concurrently using a ThreadPoolExecutor.
+    Returns a tuple of (return_code_cmd1, return_code_cmd2).
+    """
+    with concurrent.futures.ThreadPoolExecutor(max_workers=len(commands)) as executor:
+        futures = []
+        for command, log_stream_name in zip(
+            commands, log_stream_names or [None] * len(commands)
+        ):
+            futures.append(
+                executor.submit(
+                    run_single_command_non_blocking,
+                    command,
+                    user,
+                    env_vars,
+                    log_stream_name,
+                )
+            )
+        results = [
+            future.result() for future in concurrent.futures.as_completed(futures)
+        ]
+        return 0 if all(code == 0 for code in results) else 1
 
 
 def main():
@@ -132,7 +201,7 @@ def main():
             "DATATAILR_BATCH_ID": batch_id,
             "DATATAILR_BATCH_ENTRYPOINT": entrypoint,
         } | env
-        run_command_as_user("datatailr_run_batch", user, env)
+        run_single_command_non_blocking("datatailr_run_batch", user, env)
     elif job_type == "service":
         port = get_env_var("DATATAILR_SERVICE_PORT", 8080)
         entrypoint = get_env_var("DATATAILR_ENTRYPOINT")
@@ -140,15 +209,16 @@ def main():
             "DATATAILR_ENTRYPOINT": entrypoint,
             "DATATAILR_SERVICE_PORT": port,
         } | env
-        run_command_as_user("datatailr_run_service", user, env)
+        run_single_command_non_blocking("datatailr_run_service", user, env)
     elif job_type == "app":
         entrypoint = get_env_var("DATATAILR_ENTRYPOINT")
         env = {
             "DATATAILR_ENTRYPOINT": entrypoint,
         } | env
-        run_command_as_user("datatailr_run_app", user, env)
+        run_single_command_non_blocking("datatailr_run_app", user, env)
     elif job_type == "excel":
         host = get_env_var("DATATAILR_HOST", "")
+        local = get_env_var("DATATAILR_LOCAL", "")
         entrypoint = get_env_var("DATATAILR_ENTRYPOINT")
         local = get_env_var("DATATAILR_LOCAL", False)
         env = {
@@ -156,25 +226,35 @@ def main():
             "DATATAILR_HOST": host,
             "DATATAILR_LOCAL": local,
         } | env
-        run_command_as_user("datatailr_run_excel", user, env)
-    elif job_type == "ide":
-        command = [
+        run_single_command_non_blocking("datatailr_run_excel", user, env)
+    elif job_type == "workspace":
+        # Set a custom PS1 for the IDE terminal: 17:38|user@my-ide/~/dir/path:$
+        env["PS1"] = (
+            r"""\[\e[2m\]\A\[\e[0m\]|\[\e[38;5;40m\]\u\[\e[92m\]@${DATATAILR_JOB_NAME:-datatailr}\[\e[0m\]/\[\e[94;1m\]\w\[\e[0m\]\$"""
+        )
+        os.makedirs("/opt/datatailr/var/log", exist_ok=True)
+        ide_command = [
             "code-server",
             "--auth=none",
-            "--bind-addr=0.0.0.0:8080",
+            "--bind-addr=0.0.0.0:9090",
             f'--app-name="Datatailr IDE {get_env_var("DATATAILR_USER")}"',
         ]
-        run_command_as_user(command, user, env)
-    elif job_type == "jupyter":
-        command = [
+        job_name = get_env_var("DATATAILR_JOB_NAME")
+        jupyter_command = [
             "jupyter-lab",
             "--ip='*'",
-            "--port=8080",
+            "--port=7070",
             "--no-browser",
             "--NotebookApp.token=''",
             "--NotebookApp.password=''",
+            f"--ServerApp.base_url=/workspace/{job_name}/jupyter/",
+            f"--ServerApp.static_url_prefix=/workspace/{job_name}/jupyter/static/",
+            f"--ServerApp.root_dir=/home/{user}",
         ]
-        run_command_as_user(command, user, env)
+        run_commands_in_parallel(
+            [ide_command, jupyter_command], user, env, ["code-server", "jupyter"]
+        )
+
     else:
         raise ValueError(f"Unknown job type: {job_type}")
 
