@@ -13,7 +13,8 @@
 import os
 import sys
 import runpy
-from importlib.resources import files
+import inspect
+import importlib
 
 from datatailr.logging import DatatailrLogger
 
@@ -29,9 +30,23 @@ def run():
             "Environment variable 'DATATAILR_ENTRYPOINT' is not in the format 'module_name:file_name'."
         )
 
-    module_name, file_name = entrypoint.split(":")
+    module_name, func_name = entrypoint.split(":")
 
-    script = files(module_name).joinpath(file_name)
-    sys.argv = ["streamlit", "run", str(script), *sys.argv[1:]]
+    # consider using ast
+    module = importlib.import_module(module_name)
+    function = getattr(module, func_name)
+    script = inspect.getfile(function)
+    sys.argv = [
+        "streamlit",
+        "run",
+        str(script),
+        "--server.address",
+        "0.0.0.0",
+        "--server.port",
+        "8080",
+        "--server.headless",
+        "true",
+        *sys.argv[1:],
+    ]
     logger.info(f"Running entrypoint: {entrypoint}")
     runpy.run_module("streamlit", run_name="__main__")
